@@ -51,7 +51,7 @@ let has_prefix s pref =
 let ocamlpath = ref [];;
 let ocamlstdlib = ref "";;
 
-let conf_ignore_dups_in = ref (None : string option)
+let conf_ignore_dups_in = ref ([] : string list)
 
 let store = Fl_metastore.create();;
   (* We collect here only nodes, but no relations. First copy [store]
@@ -154,7 +154,7 @@ let packages_in_meta_file ?(directory_required = false)
       Failure s ->
 	close_in ch;
 	failwith ("While parsing '" ^ meta_file ^ "': " ^ s)
-    | Stream.Error s ->
+    | Fl_metascanner.Error s ->
 	close_in ch;
 	failwith ("While parsing '" ^ meta_file ^ "': " ^ s)
     | any ->
@@ -451,11 +451,12 @@ let in_report_search_path identify_dir d =
   (* Whether package dir d is to be considered for generating reports.
      d is sorted out when the ignore_dups_in option is set
    *)
-  match !conf_ignore_dups_in with
-    | None -> true
-    | Some id ->
-	try identify_dir d <> identify_dir id 
-	with _ -> Fl_split.norm_dir d <> Fl_split.norm_dir id
+  List.for_all
+    (fun id ->
+      try identify_dir d <> identify_dir id
+      with _ -> Fl_split.norm_dir d <> Fl_split.norm_dir id
+    )
+    !conf_ignore_dups_in
 ;;
 
 
@@ -530,11 +531,17 @@ let package_conflict_report ?identify_dir () =
     | Some f -> package_conflict_report_1 f ()
 ;;
 
+let check_prefix ?prefix f =
+  match prefix with
+  | None -> true
+  | Some prefix ->
+    let len = String.length prefix in
+    String.length f >= len && String.sub f 0 len = prefix
 
-let load_base() =
+let load_base ?prefix () =
   (* Ensures that the cache is completely filled with every package
-   * of the system
-   *)
+   * of the system that match prefix
+  *)
   let list_directory d =
     try
       Array.to_list(Sys.readdir d)
@@ -569,9 +576,11 @@ let load_base() =
     match path with
       [] -> ()
     | dir :: path' ->
-	let files = list_directory dir in
+        let files = list_directory dir in
 	List.iter
-	  (fun f ->
+          (fun f ->
+             if check_prefix ?prefix f
+             then
 	     (* If f/META exists: Add package f *)
 	     let package_dir = Filename.concat dir f in
 	     let meta_file_1 = Filename.concat package_dir "META" in
@@ -595,14 +604,16 @@ let load_base() =
 ;;
 
 
-let list_packages() =
-  load_base();
+let list_packages ?prefix () =
+  load_base ?prefix ();
 
   let l = ref [] in
 
   Fl_metastore.iter_up
     (fun m ->
-      l := m.package_name :: !l)
+       if check_prefix ?prefix m.package_name then
+         l := m.package_name :: !l
+    )
     store;
 
   !l
