@@ -2027,8 +2027,13 @@ let char_lowercase_ascii c =
   then Char.unsafe_chr(Char.code c + 32)
   else c
 
-let string_lowercase_ascii =
-  String.map char_lowercase_ascii
+let string_lowercase_ascii s =
+  let n = String.length s in
+  let b = Bytes.create n in
+  for i = 0 to n - 1 do
+    Bytes.unsafe_set b i (char_lowercase_ascii (String.unsafe_get s i))
+  done;
+  Bytes.to_string b
 
 
 let install_package () =
@@ -2432,7 +2437,7 @@ let print_configuration() =
   in
 
   let var = ref None in
-  let errmsg = "usage: ocamlfind printconf (conf|path|destdir|metadir|effmetadir|stdlib|ldconf)" in
+  let errmsg = "usage: ocamlfind printconf (conf|path|destdir|metadir|metapath|stdlib|ldconf)" in
 
   parse_args
         []
